@@ -1,8 +1,5 @@
-'use strict';
+import {anyMap, producersMap, eventsMap} from './maps.js';
 
-const anyMap = new WeakMap();
-const eventsMap = new WeakMap();
-const producersMap = new WeakMap();
 const anyProducer = Symbol('anyProducer');
 const resolvedPromise = Promise.resolve();
 
@@ -28,7 +25,7 @@ function assertListener(listener) {
 function getListeners(instance, eventName) {
 	const events = eventsMap.get(instance);
 	if (!events.has(eventName)) {
-		events.set(eventName, new Set());
+		return;
 	}
 
 	return events.get(eventName);
@@ -38,7 +35,7 @@ function getEventProducers(instance, eventName) {
 	const key = typeof eventName === 'string' || typeof eventName === 'symbol' || typeof eventName === 'number' ? eventName : anyProducer;
 	const producers = producersMap.get(instance);
 	if (!producers.has(key)) {
-		producers.set(key, new Set());
+		return;
 	}
 
 	return producers.get(key);
@@ -75,11 +72,18 @@ function iterator(instance, eventNames) {
 		finish() {
 			isFinished = true;
 			flush();
-		}
+		},
 	};
 
 	for (const eventName of eventNames) {
-		getEventProducers(instance, eventName).add(producer);
+		let set = getEventProducers(instance, eventName);
+		if (!set) {
+			set = new Set();
+			const producers = producersMap.get(instance);
+			producers.set(eventName, set);
+		}
+
+		set.add(producer);
 	}
 
 	return {
@@ -103,7 +107,7 @@ function iterator(instance, eventNames) {
 
 			return {
 				done: false,
-				value: await queue.shift()
+				value: await queue.shift(),
 			};
 		},
 
@@ -111,19 +115,26 @@ function iterator(instance, eventNames) {
 			queue = undefined;
 
 			for (const eventName of eventNames) {
-				getEventProducers(instance, eventName).delete(producer);
+				const set = getEventProducers(instance, eventName);
+				if (set) {
+					set.delete(producer);
+					if (set.size === 0) {
+						const producers = producersMap.get(instance);
+						producers.delete(eventName);
+					}
+				}
 			}
 
 			flush();
 
-			return arguments.length > 0 ?
-				{done: true, value: await value} :
-				{done: true};
+			return arguments.length > 0
+				? {done: true, value: await value}
+				: {done: true};
 		},
 
 		[Symbol.asyncIterator]() {
 			return this;
-		}
+		},
 	};
 }
 
@@ -162,7 +173,7 @@ function emitMetaEvent(emitter, eventName, eventData) {
 	}
 }
 
-class Emittery {
+export default class Emittery {
 	static mixin(emitteryPropertyName, methodNames) {
 		methodNames = defaultMethodNamesOrAssert(methodNames);
 		return target => {
@@ -179,14 +190,14 @@ class Emittery {
 			function getEmitteryProperty() {
 				Object.defineProperty(this, emitteryPropertyName, {
 					enumerable: false,
-					value: new Emittery()
+					value: new Emittery(),
 				});
 				return this[emitteryPropertyName];
 			}
 
 			Object.defineProperty(target.prototype, emitteryPropertyName, {
 				enumerable: false,
-				get: getEmitteryProperty
+				get: getEmitteryProperty,
 			});
 
 			const emitteryMethodCaller = methodName => function (...args) {
@@ -196,7 +207,7 @@ class Emittery {
 			for (const methodName of methodNames) {
 				Object.defineProperty(target.prototype, methodName, {
 					enumerable: false,
-					value: emitteryMethodCaller(methodName)
+					value: emitteryMethodCaller(methodName),
 				});
 			}
 
@@ -205,11 +216,13 @@ class Emittery {
 	}
 
 	static get isDebugEnabled() {
-		if (typeof process !== 'object') {
+		// eslint-disable-next-line n/prefer-global/process
+		if (typeof globalThis.process !== 'object') {
 			return isGlobalDebugEnabled;
 		}
 
-		const {env} = process || {env: {}};
+		// eslint-disable-next-line n/prefer-global/process
+		const {env} = globalThis.process || {env: {}};
 		return env.DEBUG === 'emittery' || env.DEBUG === '*' || isGlobalDebugEnabled;
 	}
 
@@ -221,6 +234,9 @@ class Emittery {
 		anyMap.set(this, new Set());
 		eventsMap.set(this, new Map());
 		producersMap.set(this, new Map());
+
+		producersMap.get(this).set(anyProducer, new Set());
+
 		this.debug = options.debug || {};
 
 		if (this.debug.enabled === undefined) {
@@ -259,7 +275,14 @@ class Emittery {
 		eventNames = Array.isArray(eventNames) ? eventNames : [eventNames];
 		for (const eventName of eventNames) {
 			assertEventName(eventName);
-			getListeners(this, eventName).add(listener);
+			let set = getListeners(this, eventName);
+			if (!set) {
+				set = new Set();
+				const events = eventsMap.get(this);
+				events.set(eventName, set);
+			}
+
+			set.add(listener);
 
 			this.logIfDebugEnabled('subscribe', eventName, undefined);
 
@@ -277,7 +300,14 @@ class Emittery {
 		eventNames = Array.isArray(eventNames) ? eventNames : [eventNames];
 		for (const eventName of eventNames) {
 			assertEventName(eventName);
-			getListeners(this, eventName).delete(listener);
+			const set = getListeners(this, eventName);
+			if (set) {
+				set.delete(listener);
+				if (set.size === 0) {
+					const events = eventsMap.get(this);
+					events.delete(eventName);
+				}
+			}
 
 			this.logIfDebugEnabled('unsubscribe', eventName, undefined);
 
@@ -321,7 +351,7 @@ class Emittery {
 
 		enqueueProducers(this, eventName, eventData);
 
-		const listeners = getListeners(this, eventName);
+		const listeners = getListeners(this, eventName) || new Set();
 		const anyListeners = anyMap.get(this);
 		const staticListeners = [...listeners];
 		const staticAnyListeners = isMetaEvent(eventName) ? [] : [...anyListeners];
@@ -337,7 +367,7 @@ class Emittery {
 				if (anyListeners.has(listener)) {
 					return listener(eventName, eventData);
 				}
-			})
+			}),
 		]);
 	}
 
@@ -350,7 +380,7 @@ class Emittery {
 
 		this.logIfDebugEnabled('emitSerial', eventName, eventData);
 
-		const listeners = getListeners(this, eventName);
+		const listeners = getListeners(this, eventName) || new Set();
 		const anyListeners = anyMap.get(this);
 		const staticListeners = [...listeners];
 		const staticAnyListeners = [...anyListeners];
@@ -401,29 +431,35 @@ class Emittery {
 			this.logIfDebugEnabled('clear', eventName, undefined);
 
 			if (typeof eventName === 'string' || typeof eventName === 'symbol' || typeof eventName === 'number') {
-				getListeners(this, eventName).clear();
+				const set = getListeners(this, eventName);
+				if (set) {
+					set.clear();
+				}
 
 				const producers = getEventProducers(this, eventName);
-
-				for (const producer of producers) {
-					producer.finish();
-				}
-
-				producers.clear();
-			} else {
-				anyMap.get(this).clear();
-
-				for (const listeners of eventsMap.get(this).values()) {
-					listeners.clear();
-				}
-
-				for (const producers of producersMap.get(this).values()) {
+				if (producers) {
 					for (const producer of producers) {
 						producer.finish();
 					}
 
 					producers.clear();
 				}
+			} else {
+				anyMap.get(this).clear();
+
+				for (const [eventName, listeners] of eventsMap.get(this).entries()) {
+					listeners.clear();
+					eventsMap.get(this).delete(eventName);
+				}
+
+				for (const [eventName, producers] of producersMap.get(this).entries()) {
+					for (const producer of producers) {
+						producer.finish();
+					}
+
+					producers.clear();
+					producersMap.get(this).delete(eventName);
+				}
 			}
 		}
 	}
@@ -434,8 +470,8 @@ class Emittery {
 
 		for (const eventName of eventNames) {
 			if (typeof eventName === 'string') {
-				count += anyMap.get(this).size + getListeners(this, eventName).size +
-					getEventProducers(this, eventName).size + getEventProducers(this).size;
+				count += anyMap.get(this).size + (getListeners(this, eventName) || new Set()).size
+					+ (getEventProducers(this, eventName) || new Set()).size + (getEventProducers(this) || new Set()).size;
 				continue;
 			}
 
@@ -471,7 +507,7 @@ class Emittery {
 
 			Object.defineProperty(target, methodName, {
 				enumerable: false,
-				value: this[methodName].bind(this)
+				value: this[methodName].bind(this),
 			});
 		}
 	}
@@ -483,13 +519,11 @@ Object.defineProperty(Emittery, 'listenerAdded', {
 	value: listenerAdded,
 	writable: false,
 	enumerable: true,
-	configurable: false
+	configurable: false,
 });
 Object.defineProperty(Emittery, 'listenerRemoved', {
 	value: listenerRemoved,
 	writable: false,
 	enumerable: true,
-	configurable: false
+	configurable: false,
 });
-
-module.exports = Emittery;
